/* Beispiele aus dem Buch "LDAP fuer Java-Entwickler", 
 * Stefan Zoerner, 4. aktualisierte Auflage 2013
 *  
 * Webseite zum Buch: http://www.ldapjava.de/ 
 * Software & Support Media GmbH / entwickler.press, http://www.entwickler-press.de/
 */
package ldapbuch.kapitel5.ad;

import java.text.ParseException;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

/**
 * Ablaufen eines Kennworts -- (Listing 5_2).
 * 
 * Test, ob und wenn ja wann ein Kennwort im AD ablaeuft. Verbindungsdaten
 * werden teilweise von aussen erwartet (jndi.properties), teilweise als
 * Kommandozeilenparameter.
 * 
 * @author Stefan Z&ouml;rner
 */
public class WannLaeuftKennwortAb {

  static String FILTER_USERS = "(&(objectClass=user)(sAMAccountName={0}))";

  /**
   * Einstiegspunkt. Als Kommandozeilenparameter werden als erste beiden Werte
   * die Credentials zur Verbindungsaufnahme erwartet, gefolgt von der Kennung
   * (sAMAccountName) des zu untersuchenden Benutzers.
   * 
   * @param args
   *          Kommandozeilenparameter
   */
  public static void main(String[] args) throws NamingException,
      ParseException {

    // Authentifizierung
    InitialDirContext ctx = new InitialDirContext();
    ctx.addToEnvironment(Context.SECURITY_AUTHENTICATION,
        "simple");
    ctx.addToEnvironment(Context.SECURITY_PRINCIPAL, args[0]);
    ctx.addToEnvironment(Context.SECURITY_CREDENTIALS, args[1]);

    // Eintrag des Benutzers suchen
    String userId = args[2];
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String[] { "pwdLastSet",
        "userAccountControl", "cn" });
    ctls.setReturningObjFlag(true);
    NamingEnumeration<SearchResult> enm = ctx.search("",
        FILTER_USERS, new String[] { userId }, ctls);

    if (!enm.hasMore()) {
      System.err.println("User nicht im Verzeichnis gefunden.");
    } else {
      SearchResult result = enm.next();
      Attributes attrs = result.getAttributes();

      System.out.println("User : " + attrs.get("cn").get());

      // (a) Laeuft das Kennwort des Benutzers ab?
      String userAccountControl = (String) attrs.get(
          "userAccountControl").get();
      int iUserAC = Integer.parseInt(userAccountControl);
      boolean laeuftNieAb = (iUserAC & AdsUserFlags.ADS_UF_DONT_EXPIRE_PASSWD) > 0;
      System.out.println("Kennwort laeuft nie ab : "
          + laeuftNieAb);

      // (b) Wann wurde das Kennwort zuletzt gesetzt?
      String pwdLastSet = (String) attrs.get("pwdLastSet").get();
      Calendar zuletztGesetzt = AdTools.hundertNanosAlsCalendar(
          "GMT", pwdLastSet);
      System.out.println("Kennwort zuletzt gesetzt : "
          + zuletztGesetzt.getTime());

      // (c) Wie lange gelten Kennwoerter in dieser Domaene
      attrs = ctx.getAttributes("", new String[] { "maxPwdAge" });
      String maxPwdAge = (String) attrs.get("maxPwdAge").get();
      int maxTage = Math.abs(AdTools
          .hundertNanosInTage(maxPwdAge));
      System.out.println("Maximales Alter von Kennworten : "
          + maxTage + " Tage");

      // (d) Berechne, in wievielen Tagen das Kennwort ablaeuft
      if (!laeuftNieAb) {
        Calendar heute = GregorianCalendar.getInstance();
        long dist = heute.getTime().getTime()
            - zuletztGesetzt.getTime().getTime();
        int alterTage = (int) (dist / (1000 * 3600 * 24));
        System.out.println("Kennwort laeuft in "
            + (maxTage - alterTage) + " Tagen ab.");
      }
    }
  }
}